/* global describe */
/* global it */
/* global expect */
import {
	addCodeInList,
	calculatePricingInfo,
	convertMoney,
	declension,
	fixImageURL,
	prefixWithZero,
	removeCodeFromList
} from '../utils';
import Money from '../schemas/Money';
import CurrencyRates from '../schemas/CurrencyRates';
import { Currency, PassengerType } from '../enums';
import { Fare } from '../schemas/PricingInfo';

const currencyRates: CurrencyRates = {
	RUB: 1,
	USD: 2,
	EUR: 4,
	GBP: 6
};

describe('utils', () => {
	describe('addCodeInList', () => {
		it('should add new string to an array', () => {
			const list: string[] = ['code_1', 'code_2'];
			const newList = addCodeInList(list, 'code_3');

			expect(!!newList.find(value => value === 'code_3')).toBe(true);
		});

		it('should not add duplicate string to an array', () => {
			const list: string[] = ['code_1', 'code_2'];
			const newList = addCodeInList(list, 'code_2');

			expect(newList.length).toBe(list.length);
		});
	});

	describe('removeCodeFromList', () => {
		it('should remove string from an array', () => {
			const list: string[] = ['code_1', 'code_2'];
			const newList = removeCodeFromList(list, 'code_2');

			expect(newList.length).toBe(list.length - 1);
		});
	});

	describe('fixImageURL', () => {
		it('should create valid URL without double slashes', () => {
			const REQUEST_URL = 'http://frontend.mlsd.ru/';
			const expectedURL = `${REQUEST_URL}image.jpg`;

			expect(fixImageURL('image.jpg', REQUEST_URL)).toBe(expectedURL);
			expect(fixImageURL('/image.jpg', REQUEST_URL)).toBe(expectedURL);
		});
	});

	describe('prefixWithZero', () => {
		it('should add zero-sign before number', () => {
			expect(prefixWithZero(9)).toBe('09');
			expect(prefixWithZero(1)).toBe('01');
		});

		it('should not add zero-sign before number', () => {
			expect(prefixWithZero(11)).toBe('11');
		});
	});

	describe('convertMoney', () => {
		it('should not perform any conversion', () => {
			const money: Money = {
				amount: 100,
				currency: Currency.RUB
			};

			expect(convertMoney(money, Currency.RUB, currencyRates)).toEqual(money);
		});

		it('should perform simple conversion', () => {
			const money: Money = {
				amount: 100,
				currency: Currency.RUB
			};

			expect(convertMoney(money, Currency.USD, currencyRates)).toEqual({ amount: 200, currency: Currency.USD });
			expect(convertMoney(money, Currency.EUR, currencyRates)).toEqual({ amount: 400, currency: Currency.EUR });
		});

		it('should perform complex conversion', () => {
			const money: Money = {
				amount: 100,
				currency: Currency.EUR
			};

			expect(convertMoney(money, Currency.USD, currencyRates)).toEqual({ amount: 50, currency: Currency.USD });
			expect(convertMoney(money, Currency.GBP, currencyRates)).toEqual({ amount: 150, currency: Currency.GBP });
		});
	});

	describe('declension', () => {
		it('should return `Москва`', () => {
			expect(declension('Москва')).toBe('Москве');
		});

		it('should return `Казани`', () => {
			expect(declension('Казань')).toBe('Казани');
		});

		it('should return `Клинцах`', () => {
			expect(declension('Клинцы')).toBe('Клинцах');
		});
	});

	describe('calculatePricingInfo', () => {
		it('should calculate taxes', () => {
			const fares: Fare[] = [];

			const fareSimple: Fare = {
				type: PassengerType.Adult,
				baseFare: {
					amount: 85,
					currency: Currency.RUB
				},
				totalFare: {
					amount: 78,
					currency: Currency.RUB
				},
				equivFare: {
					amount: 21,
					currency: Currency.RUB
				},
				count: 1,
				taxes: [],
				tariffs: [],
				taxesArray: [
					{
						code: 'XX',
						price: {
							amount: 45,
							currency: Currency.RUB
						}
					},
					{
						code: 'XY',
						price: {
							amount: 85,
							currency: Currency.RUB
						}
					}
				]
			};

			fares.push(fareSimple);
			fares.push(fareSimple);

			const expected = expect(calculatePricingInfo(fares));

			const result = fareSimple;

			result.baseFare.amount = 170;
			result.totalFare.amount = 156;
			result.equivFare.amount = 42;
			result.taxesArray.push(result.taxesArray[0]);
			result.taxesArray.push(result.taxesArray[1]);

			expected.toEqual([result]);
		});
	});
});
