import * as React from 'react';
import * as classnames from 'classnames';
import autobind from 'autobind-decorator';

import Filters from './Flight/Filters';
import FlightModel from '../models/Flight';
import SegmentModel from '../schemas/Segment';
import Details from './Flight/Details';
import Info from './Flight/Info';
import Logo from './Flight/Logo';
import RouteInfo from './Flight/RouteInfo';
import ProDetails from './Flight/ProDetails';
import { AdvancedFlightInfo } from '../schemas/SelectedFlight';
import Money from '../schemas/Money';
import FareRules from './Flight/FareRules';

export interface Props {
	flight: FlightModel;
	replacementFlight?: FlightModel;
	renderActionBlock?: () => React.ReactNode;
	renderDetails?: () => React.ReactNode;
	className?: string;
	isToggleable?: boolean;
	showFilters?: boolean;
	showOpenedSummary?: boolean;
	showFareRules?: boolean;
	showDetails?: boolean;
	nemoURL: string;
	replacedFlightInfo?: AdvancedFlightInfo;
	isLastLeg?: boolean;
	isProMode?: boolean;
	totalPrice?: Money;
}

interface State {
	isOpen: boolean;
}

const stateByFlights: { [flightId: string]: State } = {};

class Flight extends React.Component<Props, State> {
	static defaultProps: Partial<Props> = {
		isToggleable: true,
		showFilters: false,
		showOpenedSummary: false,
		showDetails: false,
		className: 'flight'
	};

	protected segmentsForDetails: SegmentModel[] = [];

	constructor(props: Props) {
		super(props);

		this.segmentsForDetails = this.props.flight.segments.slice(1);

		if (this.props.showDetails) {
			this.state = { isOpen: true };
		}
		else {
			this.state = stateByFlights[this.props.flight.id] ? stateByFlights[this.props.flight.id] : { isOpen: false };
		}
	}

	@autobind
	toggleDetails(): void {
		if (this.props.isToggleable) {
			this.setState({
				isOpen: !this.state.isOpen
			} as State);
		}
	}

	componentWillUnmount(): void {
		stateByFlights[this.props.flight.id] = this.state;
	}

	renderSummary(): React.ReactNode {
		const flight: FlightModel = this.props.flight;
		const expandClassNames = classnames('flight-summary-expand', { 'flight-summary-expand_open': this.state.isOpen });
		const classNames = classnames('flight-summary', {
			'flight-summary_open': this.state.isOpen,
			'flight-summary_isToggleable': this.props.isToggleable
		});

		return <div className={classNames} onClick={this.toggleDetails}>
			<div className="flight-summary__left">
				<div className={expandClassNames}>
					<svg fill="rgba(0, 0, 0, 0.54)" height="48" viewBox="0 0 24 24" width="48" xmlns="http://www.w3.org/2000/svg">
						<path d="M7.41 7.84L12 12.42l4.59-4.58L18 9.25l-6 6-6-6z"/>
						<path d="M0-.75h24v24H0z" fill="none"/>
					</svg>
				</div>

				<Logo flight={flight} isOpen={this.state.isOpen} nemoURL={this.props.nemoURL}/>

				<RouteInfo flight={flight} isOpen={this.state.isOpen}/>
			</div>

			<div className="flight-summary__middle">
				<Info flight={flight} isOpen={this.state.isOpen || this.props.showOpenedSummary} isProMode={this.props.isProMode}/>
			</div>

			{this.props.renderActionBlock && this.props.renderActionBlock()}
		</div>;
	}

	renderDetails(): React.ReactNode {
		return this.props.renderDetails ? this.props.renderDetails() : (
			<Details segments={this.segmentsForDetails} nemoURL={this.props.nemoURL} isProMode={this.props.isProMode}/>
		);
	}

	render(): React.ReactNode {
		const classNames = classnames(this.props.className, { flight_open: this.state.isOpen });

		return (
			<div className={classNames} data-flight-id={this.props.flight.id}>
				{this.renderSummary()}

				{this.state.isOpen && this.renderDetails()}

				{this.state.isOpen && (this.props.showFilters || this.props.showFareRules) && <div className="flight-details-info">
					{this.props.showFilters && <Filters flight={this.props.flight}/>}

					{this.props.showFareRules && <div className="flight-details-info__fareRules">
						<FareRules flight={this.props.replacementFlight ? this.props.replacementFlight : this.props.flight}/>
					</div>}
				</div>}

				{this.state.isOpen && this.props.isLastLeg && this.props.isProMode &&
					<ProDetails
						flight={this.props.flight}
						replacedFlightInfo={this.props.replacedFlightInfo}
						totalPrice={this.props.totalPrice}
					/>}
			</div>
		);
	}
}

export default Flight;
