import * as React from 'react';
import Button from '@material-ui/core/Button/Button';
import Flight from '../../models/Flight';
import { loadFareRules } from '../../store/fareRules/actions';
import { connect } from 'react-redux';
import { RootState } from '../../store/reducers';
import { getFareRules } from '../../store/fareRules/selectors';
import { FareRulesState } from '../../store/fareRules/reducers';
import DialogMessage from '../DialogMessage';
import autobind from 'autobind-decorator';
import AppBar from '@material-ui/core/AppBar/AppBar';
import Tabs from '@material-ui/core/Tabs/Tabs';
import Tab from '@material-ui/core/Tab/Tab';
import { PassengerType } from '../../enums';
import Select from '@material-ui/core/Select/Select';
import MenuItem from '@material-ui/core/MenuItem/MenuItem';
import LinearProgress from '@material-ui/core/LinearProgress/LinearProgress';
import Toolbar from '@material-ui/core/Toolbar/Toolbar';
import Typography from '@material-ui/core/Typography/Typography';
import { i18n } from '../../i18n';
import * as classnames from 'classnames';
import SwipeableViews from 'react-swipeable-views';
import { TariffRules } from '../../schemas/FareRules';

export interface OwnProps {
	flight: Flight;
}

export interface StateProps {
	fareRules: FareRulesState;
}

interface DispatchProps {
	loadFareRules: typeof loadFareRules;
}

interface State {
	popupIsOpen: boolean;
	openedSegment: number;
	passengerType: PassengerType;
}

type Props = OwnProps & StateProps;

class FareRules extends React.Component<Props & DispatchProps, State> {
	state: State = {
		popupIsOpen: false,
		openedSegment: 0,
		passengerType: PassengerType.Adult
	};

	@autobind
	onPassengerChange(event: React.ChangeEvent<HTMLSelectElement>): void {
		this.setState({ passengerType: event.target.value as PassengerType });
	}

	@autobind
	onChangeTab(event: React.ChangeEvent, value: number): void {
		this.setState({ openedSegment: value });
	}

	@autobind
	toggleTranslate(): void {
		this.props.loadFareRules(this.props.flight.id, !this.props.fareRules[this.props.flight.id].isTranslated);
	}

	@autobind
	closePopup(): void {
		this.setState({ popupIsOpen: false });
	}

	@autobind
	onClick(): void {
		if (!this.props.fareRules.hasOwnProperty(this.props.flight.id)) {
			this.props.loadFareRules(this.props.flight.id);
		}

		this.setState({ popupIsOpen: true });
	}

	renderPassengerSelector(): React.ReactNode {
		return <div className="fareRules__typeChooser">
			<span>
				{i18n('fareRules-for-title')}
			</span>

			<Select onChange={this.onPassengerChange} value={this.state.passengerType}>
				{Object.keys(PassengerType).map((type, index) => {
					const passengerType = PassengerType[type as any];

					if (this.props.fareRules[this.props.flight.id].fareRulesByPassenger[0].hasOwnProperty(passengerType)) {
						return <MenuItem value={passengerType} key={index}>
							{i18n(`fareRules-for_${passengerType}`)}
						</MenuItem>;
					}
				})}
			</Select>
		</div>;
	}

	renderSegment(tariffRules: TariffRules[]): React.ReactNode {
		return <div>
			<div className="fareRules-segment-tariffCode">
				<span>
					{i18n('fareRules-tariffCode')}
				</span>

				<strong>{tariffRules[0].tariffCode}</strong>
			</div>

			{tariffRules.map((rules, index) => {
				return <div className="fareRules-segment-rule" key={index}>
					<strong className="fareRules-segment-rule__name">{`${rules.code}. ${rules.name}`}</strong>
					<div className="fareRules-segment-rule__text">{rules.text}</div>
				</div>;
			})}
		</div>;
	}

	renderSegmentTabs(): React.ReactNode {
		const { flight } = this.props;

		return <AppBar position="static" className={classnames('fareRules-tabs', {'fareRules-tabs_oneSegment': flight.segments.length <= 1})}>
			<Tabs value={this.state.openedSegment} onChange={this.onChangeTab}>
				{flight.segments.length > 1 && flight.segments.map((segment, index) =>
					<Tab value={index} key={index} label={`${segment.depAirport.city.name} — ${segment.arrAirport.city.name}`}/>
				)}
			</Tabs>
		</AppBar>;
	}

	renderDummy(): React.ReactNode {
		const blocks: React.ReactNode[] = [];

		for (let index = 0; index < 4; index++) {
			blocks.push(<div className="fareRules-dummy__rule" key={index}>
				<div className="fareRules-dummy__header"/>

				<div className="fareRules-dummy__text">∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎
					∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎
					∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎∎</div>
			</div>);
		}

		return <div className="fareRules-loading">
			<LinearProgress variant="query" />

			<div className="fareRules-loading__header">
				{i18n('fareRules-loading')}
			</div>

			<div className="fareRules-dummy">
				{blocks}
			</div>
		</div>;
	}

	renderHeader(): React.ReactNode {
		const { flight, fareRules } = this.props;

		return <Toolbar className="fareRules__header">
			<Typography variant="title">{i18n('fareRules-title')}</Typography>

			{fareRules && fareRules[flight.id] && fareRules[flight.id].canBeTranslated &&
				<Button onClick={this.toggleTranslate}>
					{i18n(fareRules[flight.id].isTranslated ? 'fareRules-translate_back' : 'fareRules-translate')}
				</Button>}
		</Toolbar>;
	}

	renderContent(): React.ReactNode {
		const { flight, fareRules } = this.props;

		if (!fareRules || !fareRules.hasOwnProperty(flight.id)) {
			return null;
		}

		if (fareRules.hasOwnProperty(flight.id) && fareRules[flight.id] === null) {
			return <>
				{this.renderHeader()}
				{this.renderDummy()}
			</>;
		}
		else if (fareRules[flight.id].manualRulesArray) {
			return <>
				{this.renderHeader()}

				<div className="fareRules-segment">
					<div className="fareRules-segment__wrapper">
						<div>
							{fareRules[flight.id].manualRulesArray.map((rule, index) => (
								<div key={index} className="fareRules-segment-rule">
									<div className="fareRules-segment-rule__text">{rule}</div>
								</div>
							))}
						</div>
					</div>
				</div>
			</>;
		}
		else if (fareRules[flight.id].fareRulesByPassenger.length) {
			return <>
				{this.renderHeader()}
				{this.renderSegmentTabs()}

				<div className="fareRules-segment">
					{this.renderPassengerSelector()}

					<SwipeableViews index={this.state.openedSegment}>
						{fareRules[flight.id].fareRulesByPassenger.map((fareRulesSegment, index) => {
							return <div className="fareRules-segment__wrapper" key={index}>
								{this.renderSegment(fareRulesSegment[this.state.passengerType])}
							</div>;
						})}
					</SwipeableViews>
				</div>
			</>;
		}
		else {
			return (
				<div className="fareRules-error">
					<div className="fareRules-error__header">
						<Typography variant="title">
							{i18n('fareRules-error-title')}
						</Typography>
					</div>

					<div className="fareRules-error__text">
						{i18n('fareRules-error-text')}
					</div>
				</div>
			);
		}
	}

	renderActions(): React.ReactNode {
		return <Button onClick={this.closePopup}>
			{i18n('fareRules-close')}
		</Button>;
	}

	render(): React.ReactNode {
		return <>
			<Button onClick={this.onClick}>
				{i18n('fareRules-button-title')}
			</Button>

			<DialogMessage
				visible={this.state.popupIsOpen}
				content={this.renderContent()}
				actions={this.renderActions()}
				dialogClass="fareRules__dialog"
				onClose={this.closePopup}
			/>
		</>;
	}
}

const mapStateToProps = (state: RootState): StateProps => {
	return {
		fareRules: getFareRules(state)
	};
};

const mapDispatchToProps = {
	loadFareRules
};

export default connect(mapStateToProps, mapDispatchToProps)(FareRules);
