import * as moment from 'moment';
import FlightSchema from '../schemas/Flight';
import Airline from '../schemas/Airline';
import SegmentGroup from '../schemas/SegmentGroup';
import Segment from '../schemas/Segment';
import Money from '../schemas/Money';
import { convertNemoDateToMoment, declension, pluralize, UID_LEG_GLUE, UID_SEGMENT_GLUE } from '../utils';
import Date from '../schemas/Date';
import Fillable from '../models/Fillable';
import { i18n } from '../i18n';
import { ObjectsMap } from '../store/filters/selectors';
import FareFamily from '../schemas/FareFamily';
import FareFamilyFeature, { FeatureCode, FeaturePayment } from '../schemas/FareFamilyFeature';
import StopPoint from '../schemas/StopPoint';
import { ListOfIATACodes } from '../store/selectedFlights/selectors';
import { PricingDebugLink, PricingInfo, TaxInfo } from '../schemas/PricingInfo';

const getBaggageFromFamily = (family: FareFamily): FareFamilyFeature => {
	let result;

	result = family.fareFeatures.baggage.find(feature => feature.name === FeatureCode.BAGGAGE);

	return result;
};

export default class Flight extends Fillable<FlightSchema> {
	id: string;
	altFlightHasBeenChosen: boolean;
	altFlights: Flight[];
	codeShareAirlines: Airline[];
	isInternational: boolean;
	isRefundable: boolean;
	isTranslitRequired: boolean;
	originalCurrency: string;
	searchId: number;
	segmentGroups: SegmentGroup[];
	segments: Segment[];
	service: string;
	serviceFltId: string;
	totalPrice: Money;
	validatingCarrier: Airline;
	searchIndex: string;
	baggageIsFree: boolean;

	uid: string; // XX767_1pc__XX767_2pc
	totalFlightTime: number;
	totalFlightTimeHuman: string;
	arrivalAtNextDay: boolean;
	firstSegment: Segment;
	lastSegment: Segment;
	transferInfo: string;
	stopoversInfo: string;
	isRT: boolean = false;
	legId: number;
	airlinesList: Airline[];
	carriersMismatchLabel: string;
	expectedNumberOfTickets: number;
	pricingDebugLink: PricingDebugLink;
	pricingInfo: PricingInfo;

	fill(flightSource: FlightSchema): void {
		super.fill(flightSource);

		const UID: string[] = [];
		const airlinesMap: ObjectsMap<Airline> = {};
		const airlinesInFlight: Airline[] = [];
		const carrierNamesMismatch: string[] = [];
		const carriersMismatchMap: ListOfIATACodes = {};
		let stopPoints: StopPoint[] = [];
		let totalFlightTime = 0;

		this.id = flightSource.id.toString();
		this.baggageIsFree = true;

		this.segmentGroups = flightSource.segmentGroups.map(group => {
			const legUID: string[] = [];

			group.segments = group.segments.map((segment: any): Segment => {
				// Convert dates from Nemo object to a Moment.js object.
				if ('offsetUTC' in segment.arrDate) {
					segment.arrDate = convertNemoDateToMoment(<Date>segment.arrDate);
				}

				if ('offsetUTC' in segment.depDate) {
					segment.depDate = convertNemoDateToMoment(<Date>segment.depDate);
				}

				const parsedSegment = segment as Segment;
				let baggageFromFamily: FareFamilyFeature;

				// Generate UID for each segment.
				legUID.push(this.createSegmentUID(parsedSegment));

				// Sum up total flight time.
				totalFlightTime += parsedSegment.flightTime + parsedSegment.waitingTime;

				// Pull airline object from segment.
				if (!airlinesMap.hasOwnProperty(parsedSegment.airline.IATA)) {
					airlinesMap[parsedSegment.airline.IATA] = parsedSegment.airline;
					airlinesInFlight.push(parsedSegment.airline);
				}

				if (parsedSegment.fareFamilyFeatures && parsedSegment.fareFamilyFeatures.isLoaded) {
					baggageFromFamily = getBaggageFromFamily(parsedSegment.fareFamilyFeatures);
				}

				// Check if luggage is free.
				if (baggageFromFamily) {
					this.baggageIsFree = baggageFromFamily.needToPay === FeaturePayment.Free;
				}
				else if (!parsedSegment.luggage || !parsedSegment.luggage.value) {
					this.baggageIsFree = false;
				}

				if (parsedSegment.stopPoints && parsedSegment.stopPoints.length) {
					stopPoints = stopPoints.concat(parsedSegment.stopPoints);
				}

				if (
					parsedSegment.operatingAirline.IATA !== parsedSegment.airline.IATA &&
					!carriersMismatchMap.hasOwnProperty(parsedSegment.operatingAirline.IATA)
				) {
					carrierNamesMismatch.push(parsedSegment.operatingAirline.name);
					carriersMismatchMap[parsedSegment.operatingAirline.IATA] = parsedSegment.operatingAirline.name;
				}

				parsedSegment.baggageFromFamily = baggageFromFamily;

				return parsedSegment;
			});

			// Glue together all segment UIDs.
			UID.push(legUID.join(UID_SEGMENT_GLUE));

			return group;
		});

		// Glue together all leg UIDs.
		this.uid = UID.join(UID_LEG_GLUE);

		if (carrierNamesMismatch.length) {
			this.carriersMismatchLabel = `${i18n(carrierNamesMismatch.length > 1 ? 'results-flight-operatingCompanies' : 'results-flight-operatingCompany')}: ${carrierNamesMismatch.join(', ')}`;
		}
		this.airlinesList = airlinesInFlight;
		this.totalFlightTime = totalFlightTime;
		this.totalFlightTimeHuman = moment.duration(totalFlightTime, 'seconds').format(`d [${i18n('utils-dates-d')}] h [${i18n('utils-dates-h')}] m [${i18n('utils-dates-m')}]`);

		this.firstSegment = this.segments[0];
		this.lastSegment = this.segments[this.segments.length - 1];
		this.arrivalAtNextDay = this.firstSegment.depDate.date() !== this.lastSegment.arrDate.date();

		const numOfTransfers = this.segments.length - 1;

		if (numOfTransfers === 1) {
			this.transferInfo = `${i18n('results-flight-transfersTitle_1')} ${i18n('utils-pre-in')} ${declension(this.segments[0].arrAirport.city.name)}`;
		}
		else {
			const pluralizedTitle = pluralize(numOfTransfers, i18n('results-flight-transfersTitle_1'), i18n('results-flight-transfersTitle_2'), i18n('results-flight-transfersTitle_3'));
			this.transferInfo = `${numOfTransfers} ${pluralizedTitle}`;
		}

		if (stopPoints.length === 1) {
			this.stopoversInfo = `${i18n('results-flight-stopoverTitle_one')} ${declension(stopPoints[0].city ? stopPoints[0].city.name : stopPoints[0].airport.name)}`;
		}
		else if (stopPoints.length > 1) {
			this.stopoversInfo = i18n('results-flight-stopoverTitle_several');
		}

		this.searchIndex = this.uid.toLowerCase() +
			this.validatingCarrier.name.toLowerCase() +
			this.validatingCarrier.nameEn.toLowerCase() +
			this.firstSegment.depAirport.IATA.toLowerCase() +
			this.firstSegment.depAirport.name.toLowerCase() +
			this.firstSegment.depAirport.city.name.toLowerCase() +
			this.lastSegment.arrAirport.IATA.toLowerCase() +
			this.lastSegment.arrAirport.name.toLowerCase() +
			this.lastSegment.arrAirport.city.name.toLowerCase();

		this.pricingInfo.fares.forEach(fare => {
			const flatTaxInfo: TaxInfo = {};

			fare.taxesArray = [];

			fare.taxes.forEach(tax => {
				for (const item in tax) {
					if (tax.hasOwnProperty(item)) {
						if (flatTaxInfo.hasOwnProperty(item)) {
							flatTaxInfo[item].amount += tax[item].amount;
						}
						else {
							flatTaxInfo[item] = tax[item];
						}
					}
				}
			});

			for (const item in flatTaxInfo) {
				if (flatTaxInfo.hasOwnProperty(item)) {
					fare.taxesArray.push({
						code: item,
						price: flatTaxInfo[item]
					});
				}
			}
		});

		if (this.pricingDebugLink) {
			this.pricingDebugLink.route = this.firstSegment.depAirport.city ? this.firstSegment.depAirport.city.name : this.firstSegment.depAirport.name;
			this.pricingDebugLink.route += ' - ';
			this.pricingDebugLink.route += this.lastSegment.arrAirport.city ? this.lastSegment.arrAirport.city.name : this.lastSegment.arrAirport.name;
		}
	}

	/**
	 * Create a part of flight Unique ID (UID) for each segment.
	 *
	 * XX-123_1pc (flight number on the segment + free baggage allowance on the segment).
	 *
	 * @param {Segment} segment
	 * @returns {string}
	 */
	protected createSegmentUID(segment: Segment): string {
		return `${segment.airline.IATA}-${segment.flightNumber}_${segment.luggage.value === null ? 0 : segment.luggage.value}${segment.luggage.measurement}`;
	}
}
