import { SearchInfoPassenger } from '@nemo.travel/search-widget';
import { PassengerType } from '../../enums';
import * as moment from 'moment';

export interface FastSearchSegment {
	departureIATA: string;
	arrivalIATA: string;
	departureDate: moment.Moment;
	returnDate?: moment.Moment;
}

export interface FastSearchRequest {
	segments: FastSearchSegment[];
	passengers: {
		[type: string]: number;
	}
	commands: {
		[name: string]: FastSearchCommand;
	}
}

export interface FastSearchCommand {
	name: string;
	value: string;
}

export class FastSearch {
	fastSearchString = '';

	constructor(location: string) {
		this.fastSearchString = location.substr(9);
	}

	isIATANext(): boolean {
		return /[A-ZА-Я]{3}(\d{8,16}|[A-ZА-Я]{3})/.test(this.fastSearchString);
	}

	isDateNext(): boolean {
		return /\d{8}/.test(this.fastSearchString.substr(0, 8));
	}

	isPassengerNext(): boolean {
		return /[A-Z]\d/.test(this.fastSearchString.substr(0, 4));
	}

	isCommandNext(): boolean {
		return this.fastSearchString[0] === '-';
	}

	getNextIATA(): string {
		const data = this.fastSearchString.substr(0, 3);

		this.fastSearchString = this.fastSearchString.substr(3);

		return data;
	}

	getNextDate(): moment.Moment {
		const date = this.fastSearchString.substr(0, 8);

		this.fastSearchString = this.fastSearchString.substr(8);

		return moment(date, 'YYYYMMDD');
	}

	getNextPassenger(): SearchInfoPassenger {
		const passengerType = this.fastSearchString.substr(0, 3),
			passegnersTypeCount = this.fastSearchString.substr(3, 1);

		this.fastSearchString = this.fastSearchString.substr(4);

		return {
			type: passengerType as PassengerType,
			count: parseInt(passegnersTypeCount)
		};
	}

	getNextCommand(): FastSearchCommand {
		const commandGroup = this.fastSearchString.match(/-([a-zA-Z]+)=([a-zA-Z]+)/);

		this.fastSearchString = this.fastSearchString.replace(commandGroup[0], '');

		return {
			name: commandGroup[1],
			value: commandGroup[2]
		};
	}

	parse(): FastSearchRequest {
		const fastSearch: FastSearchRequest = {
			segments: [],
			passengers: {},
			commands: {}
		};

		while(this.isIATANext()) {
			fastSearch.segments.push({
				departureIATA: this.getNextIATA(),
				arrivalIATA: this.getNextIATA(),
				departureDate: this.getNextDate(),
				returnDate: null
			});

			if (this.isDateNext()) {
				const lastSegment = fastSearch.segments[fastSearch.segments.length - 1];

				fastSearch.segments.push({
					departureIATA: lastSegment.arrivalIATA,
					arrivalIATA: lastSegment.departureIATA,
					departureDate: this.getNextDate(),
					returnDate: null
				});
			}
		}

		while(this.isPassengerNext()) {
			const passenger = this.getNextPassenger();

			fastSearch.passengers[passenger.type] = passenger.count;
		}

		while(this.isCommandNext()) {
			const command = this.getNextCommand();

			fastSearch.commands[command.name] = command;
		}

		return fastSearch;
	}
}
