import * as moment from 'moment';
import { createStructuredSelector, Selector } from 'reselect';
import { connect as reduxConnect, InferableComponentEnhancerWithProps, MapDispatchToPropsParam } from 'react-redux';
import { SearchInfo, SearchInfoSegment } from '@nemo.travel/search-widget';

import { Currency, Language, RouteType } from './enums';
import Date from './schemas/Date';
import RequestInfo from './schemas/RequestInfo';
import Leg from './schemas/Leg';
import { RootState } from './store/reducers';
import Money from './schemas/Money';
import CurrencyRates from './schemas/CurrencyRates';
import { Fare } from './schemas/PricingInfo';
import SearchRequests from './schemas/SearchRequests';
import { ResultsState } from './store/results/reducers';

export const UID_LEG_GLUE = '|';
export const UID_SEGMENT_GLUE = '_';
export const ISO_DATE_LENGTH = 19;
export const MAX_VISIBLE_FLIGHTS = 15;
export const NUM_OF_RT_SEGMENTS = 2;
export const MIN_TRANSFER_TIME_IN_HOURS_BETWEEN_FLIGHTS = 2;

interface CurrencyRoundList {
	[currency: string]: number;
}

/**
 * Конфиг с кол-вом знаков после запятой для разных валют.
 */
export const digitsAfterPoint: CurrencyRoundList = {
	RUB: 0,
	USD: 2,
	GBP: 2,
	EUR: 2,
	CNY: 2,
	LVL: 2,
	UAH: 2,
	CZK: 2,
	JPY: 2
};

const lastPlainNumber = 9;

/**
 * Добавляет нолик к цифрам.
 * '9' -> '09'
 *
 * @param {number} plainNumber
 * @returns {string}
 */
export const prefixWithZero = (plainNumber: number): string => plainNumber >= 0 && plainNumber <= lastPlainNumber ? '0' + plainNumber.toString() : plainNumber.toString();

/**
 * Производит конвертацию объекта денег в другую валюту.
 *
 * @param {Money} money
 * @param {Currency} currency - во что конвертить
 * @param {CurrencyRates} rates - объект с курсами валют
 */
export const convertMoney = (money: Money, currency: Currency, rates: CurrencyRates): Money => {
	if (money.currency === currency) {
		return money;
	}

	const roundFactor = Math.pow(10, digitsAfterPoint[currency]);
	let amount: number;

	// Абсолютное значение.
	amount = Math.abs(money.amount);

	// Конвертируем сумму на основе данных о курсах валют.
	amount = amount * (rates[currency] / rates[money.currency]);

	// Округляем, оставляя определенное кол-во знаков после запятой.
	amount = Math.round(amount * roundFactor) / roundFactor;

	return { amount, currency };
};

/**
 * Конвертирует Немо-дату в MomentJS.
 *
 * @param {Date} date
 * @returns {moment.Moment}
 */
export const convertNemoDateToMoment = (date: Date): moment.Moment => {
	let dateString = `${date.year}-${prefixWithZero(date.month)}-${prefixWithZero(date.day)}T${prefixWithZero(date.hours)}:${prefixWithZero(date.minutes)}:${prefixWithZero(date.seconds)}`;

	if (date.offsetUTC) {
		dateString += date.offsetUTC;
	}

	return moment(dateString);
};

/**
 * Удаляет ведущие\хвостовые слэши.
 *
 * @param {String} target
 */
export const trimSlashes = (target: string): string => {
	return target.replace(/\/+$/, '').replace(/^\/+/, '');
};

/**
 * Генерит правильную ссылку на картинку.
 *
 * @param {String} imageURL
 * @param {String} nemoURL
 */
export const fixImageURL = (imageURL: string, nemoURL: string): string => {
	return nemoURL + trimSlashes(imageURL);
};

/**
 * Добавляет уникальный элемент в массив.
 *
 * @FIXME: Заменить на {Set}
 *
 * @param {String[]} list
 * @param {String} code
 */
export const addCodeInList = (list: string[], code: string): string[] => {
	const result: string[] = [...list];

	if (!list.find(existingCode => existingCode === code)) {
		result.push(code);
	}

	return result;
};

/**
 * Удаляет уникальный элемент из массива.
 *
 * @FIXME: Заменить на {Set}
 *
 * @param {String[]} list
 * @param {String} code
 */
export const removeCodeFromList = (list: string[], code: string): string[] => {
	return list.filter(existingCode => existingCode !== code);
};

/**
 * Базовый механизм склонения.
 * В первую очередь - для склонения названия городов на русском.
 *
 * @param {String} word
 * @param {Language} language
 */
export const declension = (word: string, language = Language.Russian): string => {
	if (language === Language.Russian) {
		const lastLetterIndex = word.length - 1;
		const lastLetter = word[lastLetterIndex];
		let newWord = word;
		let newSuffix: string;

		if (word === 'Казань' || word === 'Пермь') {
			newSuffix = 'и';
		}
		else if (/[аеёийоуыьэюя]/i.test(lastLetter)) {
			if (/[айь]/i.test(lastLetter)) {
				newSuffix = 'е';
			}
			else if (lastLetter === 'ы') {
				newSuffix = 'ах';
			}
		}
		else {
			newWord = word + 'е';
		}

		if (newSuffix) {
			newWord = word.substr(0, lastLetterIndex) + newSuffix;
		}

		return newWord;
	}

	return word;
};

/**
 * Генерирует формы множественного числа.
 *
 * @param {Number} number
 * @param {String[]} variations
 */
export const pluralize = (number: number, ...variations: string[]): string => {
	let result = '';
	const lastLetterIndex = number.toString().length - 1;
	const lastLetter = number.toString()[lastLetterIndex];

	switch (lastLetter) {
		case '0':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			result = variations[2];
			break;

		case '1':
			result = number === 11 ? variations[2] : variations[0];
			break;

		case '2':
		case '3':
		case '4':
			result = number > 11 && number < 14 ? variations[2] : variations[1];
			break;
	}

	return result;
};

/**
 * Создает объекты этапов\плеч.
 *
 * @param requests
 */
export const createLegs = (requests: RequestInfo[]): Leg[] => (
	requests.map((requestInfo, index) => ({
		id: index,
		departure: requestInfo.segments[0].departure,
		arrival: requestInfo.segments[0].arrival,
		date: requestInfo.segments[0].departureDate
	}))
);

type SelectorsList<P> = {
	[K in keyof P]: Selector<RootState, P[K]>
};

/**
 * Супер-шорткат для redux-connect.
 *
 * @param selectors
 * @param actions
 */
export function connect<StateProps, DispatchProps = {}, OwnProps = {}>(
	selectors: SelectorsList<StateProps>,
	actions?: MapDispatchToPropsParam<DispatchProps, OwnProps>
): InferableComponentEnhancerWithProps<StateProps & DispatchProps, OwnProps> {
	return reduxConnect(createStructuredSelector<RootState, StateProps>(selectors), actions);
}

/**
 * Суммирует инфу о тарифах.
 * Необходимо для попапа с детализацией цены.
 *
 * @param pricingInfo
 */
export const calculatePricingInfo = (pricingInfo: Fare[]): Fare[] => {
	const pricingInfoMap: { [type: string]: Fare } = {};

	pricingInfo.forEach(pricing => {
		if (!pricingInfoMap.hasOwnProperty(pricing.type)) {
			pricingInfoMap[pricing.type] = {
				...pricing,
				totalFare: {
					amount: 0,
					currency: pricing.totalFare.currency
				},
				equivFare: {
					amount: 0,
					currency: pricing.equivFare.currency
				},
				baseFare: {
					amount: 0,
					currency: pricing.baseFare.currency
				},
				taxesArray: []
			};
		}

		pricingInfoMap[pricing.type].totalFare.amount += pricing.totalFare.amount;
		pricingInfoMap[pricing.type].equivFare.amount += pricing.equivFare.amount;
		pricingInfoMap[pricing.type].baseFare.amount += pricing.baseFare.amount;
		pricingInfoMap[pricing.type].taxesArray = pricingInfoMap[pricing.type].taxesArray.concat(pricing.taxesArray);
	});

	const pricingArray: Fare[] = [];

	for (const item in pricingInfoMap) {
		if (pricingInfoMap.hasOwnProperty(item)) {
			pricingArray.push(pricingInfoMap[item]);
		}
	}

	return pricingArray;
};

/**
 * Генерирует шаблоны поисковых запросов.
 *
 * @param searchInfo информация о выбранных на форме поиска параметрах
 */
export const createSearchPayload = (searchInfo: SearchInfo): SearchRequests => {
	let throughRequest: RequestInfo = null;
	let requests: RequestInfo[] = [];

	const segments = searchInfo.segments;
	const commonParams = {
		passengers: searchInfo.passengers,
		parameters: {
			delayed: false,
			direct: searchInfo.directOnly,
			serviceClass: searchInfo.serviceClass
		}
	};

	switch (searchInfo.routeType) {
		case RouteType.RT:
			// RT search
			const departureSegment: SearchInfoSegment = segments[0];
			const returnSegment: SearchInfoSegment = segments[1];

			requests.push({ ...commonParams, segments: [departureSegment] });
			requests.push({ ...commonParams, segments: [returnSegment] });

			throughRequest = { ...commonParams, segments: [departureSegment, returnSegment] };

			break;

		case RouteType.OW:
			requests.push({
				...commonParams,
				segments: [segments[0]]
			});

			break;

		case RouteType.CR:
			requests = segments.map((segment: SearchInfoSegment): RequestInfo => ({
				...commonParams,
				segments: [segment]
			}));

			// Для ситуаций, когда на форме выбран сложный маршрут, но оставлен всего один сегмент.
			if (segments.length > 1) {
				throughRequest = { ...commonParams, segments: segments };
			}

			break;
	}

	return {
		requests,
		unionRequest: throughRequest
	};
};

/**
 * @param pathname
 */
export const createSearchResultsPayload = (pathname: string): ResultsState[] => {
	const parts = pathname.split('/').slice(2);
	const isOW = parts.length === 1;
	const lastIndex = parts.length - 1;

	return parts.filter(resultsId => !!trimSlashes(resultsId)).map((resultsId, index) => ({
		id: parseInt(trimSlashes(resultsId)),
		isRT: !isOW && lastIndex === index
	}));
};
